<!-- This file is part of deegree, for copyright/license information, please visit http://www.deegree.org/license. -->
<xsl:stylesheet version="1.0" 
    xmlns="http://www.opengis.net/context" 
    xmlns:csw="http://www.opengis.net/cat/csw/2.0.2" 
    xmlns:dcmiBox="http://dublincore.org/documents/2000/07/11/dcmi-box/" 
    xmlns:ogc="http://www.opengis.net/ogc" 
    xmlns:xlink="http://www.w3.org/1999/xlink" 
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
    xmlns:gco="http://www.isotc211.org/2005/gco" 
    xmlns:gmd="http://www.isotc211.org/2005/gmd" 
    xmlns:srv="http://www.isotc211.org/2005/srv">
    <xsl:output method="html" media-type="text/html" omit-xml-declaration="yes" encoding="UTF-8"/>
    
    <xsl:template name="getCapsTemplate">
        <xsl:for-each select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:containsOperations">
            <xsl:variable name="operation" select="./srv:SV_OperationMetadata/srv:operationName/gco:CharacterString"/>
            <xsl:if test=" $operation = 'GetCapabilities' ">                
                <xsl:variable name="serviceUrl" select="./srv:SV_OperationMetadata/srv:connectPoint/gmd:CI_OnlineResource/gmd:linkage/gmd:URL"/>
                <xsl:variable name="serviceType" select="../srv:serviceType/gco:LocalName" />
                <xsl:variable name="serviceVer" select="../srv:serviceTypeVersion/gco:CharacterString" />
                <xsl:variable name="getcapsRef" select="concat( 'javascript:buildGetCapsReq(', $serviceUrl, ', ', $serviceType, ', ', $serviceVer, ');' )"/>
                <xsl:choose>
                <xsl:when test="contains( $serviceUrl, '?' )">
                    <xsl:variable name="getCapsReq" select="concat($serviceUrl, 'REQUEST=GetCapabilities&amp;SERVICE=', $serviceUrl, '&amp;VERSION=', $serviceVer)"/>
                    <tr>
	                    <td class="key">Get Capabilities</td>
	                    <td class="val"><a class="heighlight" href="($getCapsReq)"><xsl:value-of select="$getCapsReq"/></a></td>
                    </tr>    
                </xsl:when>
                <xsl:otherwise>
                    <xsl:variable name="getCapsReq" select="concat($serviceUrl, '?', 'REQUEST=GetCapabilities&amp;SERVICE=', $serviceUrl, '&amp;VERSION=', $serviceVer)"/>
                    <tr>
	                    <td class="key">Get Capabilities:</td>
	                    <td class="val"><a class="heighlight" href="($getCapsReq)"><xsl:value-of select="$getCapsReq"/></a></td>
                    </tr>
                </xsl:otherwise>
                </xsl:choose>
                
            </xsl:if>
        </xsl:for-each>
    </xsl:template>
    <xsl:template name="DE_MD_Metadata">
        <xsl:variable name="ServLength">
            <xsl:value-of select="string-length($SERVICECATALOGS)"/>
        </xsl:variable>
        <xsl:variable name="ident">
            <xsl:value-of select="./gmd:MD_Metadata/gmd:fileIdentifier/gco:CharacterString"/>
        </xsl:variable>
        <xsl:variable name="mdType">
            <xsl:value-of select="./gmd:MD_Metadata/gmd:hierarchyLevel/gmd:MD_ScopeCode/@codeListValue"/>
        </xsl:variable>
        <xsl:variable name="quote">
            <xsl:text disable-output-escaping="yes">'</xsl:text>
        </xsl:variable>
        <xsl:variable name="geoobjecttype">
            <xsl:value-of select="./gmd:MD_Metadata/gmd:spatialRepresentationInfo/gmd:MD_VectorSpatialRepresentation/gmd:geometricObjects/gmd:MD_GeometricObjects/gmd:geometricObjectType/gmd:MD_GeometricObjectTypeCode/@codeListValue" />
        </xsl:variable>
        <!-- <xsl:variable name="args" select="concat( $quote, $ident, $quote, ', ', $quote, $title, $quote, ', ', $quote, $CATALOG, $quote, ', ', $minx, ', ', $miny, ', ', $maxx, ', ', $maxy, ', ', $quote, $SERVICECATALOGS, $quote )" />  -->
        <!-- <xsl:variable name="shoppingRef" select="concat( 'javascript:addToShoppingCart( ', $args, ' );' )"/>  -->
        <xsl:variable name="argsChildren" select="concat( $quote, $ident, $quote, ', ', $quote, $CATALOG, $quote )" />
        <xsl:variable name="childListRef" select="concat( 'javascript:performSeriesChildrenSearch( ', $argsChildren, ' );' )"/>
        <xsl:choose>
            <xsl:when test="$mdType = 'service'">
                <xsl:variable name="title">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString"/>
                </xsl:variable>
                <xsl:variable name="minx">
					<xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="maxx">
					<xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="miny">
					<xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="maxy">
					<xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal" />
				</xsl:variable>
                <xsl:variable name="serviceType">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:serviceType/gco:LocalName"/>
                </xsl:variable>
                <xsl:variable name="serviceVer">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/srv:SV_ServiceIdentification/srv:serviceTypeVersion/gco:CharacterString"/>
                </xsl:variable>
                <!-- contents -->
                <table class="md">
                    <tr>
                        <td class="key">Catalogue:</td>
                        <td class="val"><xsl:value-of select="$CATALOG"/></td>
                    </tr>
                    <!-- 
                    <tr>
						<td class="key">Topic Category</td>
						<td class="val"><xsl:value-of select="$topicCategory" /></td>
					</tr>
                    -->
                    <tr>
                        <td class="key">Identifier:</td>
                        <td class="val"><xsl:value-of select="$ident"/></td>
                    </tr>
                    <tr>
                        <td class="key">Title:</td>
                        <td class="val"><xsl:value-of select="$title"/></td>
                    </tr>
                    <tr>
                        <td class="key">Metadata Type:</td>
                        <td class="val"><xsl:value-of select="$mdType"/></td>
                    </tr>
                    <tr>
                        <td class="key">BoundingBox:</td>
                        <td class="val"><xsl:value-of select="$minx"/>, <xsl:value-of select="$miny"/>, <xsl:value-of select="$maxx"/>, <xsl:value-of select="$maxy"/></td>
                    </tr>
                    <tr>
                        <td class="key">Service Type:</td>
                        <td class="val"><xsl:value-of select="$serviceType"/></td>
                    </tr>
                    <tr>
                        <td class="key">Service Version:</td>
                        <td class="val"><xsl:value-of select="$serviceVer"/></td>
                    </tr>
                    <xsl:call-template name="getCapsTemplate"/>
                </table>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="topicCategory">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:topicCategory/gmd:MD_TopicCategoryCode"/>
                </xsl:variable>
                <xsl:variable name="title">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString"/>
                </xsl:variable>
                <xsl:variable name="minx">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal"/>
                </xsl:variable>
                <xsl:variable name="maxx">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal"/>
                </xsl:variable>
                <xsl:variable name="miny">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal"/>
                </xsl:variable>
                <xsl:variable name="maxy">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal"/>
                </xsl:variable>
                <xsl:variable name="creation_date">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:citation/gmd:CI_Citation/gmd:date/gmd:CI_Date[gmd:dateType/gmd:CI_DateTypeCode/@codeListValue='creation']/gmd:date/gco:Date" />
                </xsl:variable>
                <xsl:variable name="contact">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:pointOfContact/gmd:CI_ResponsibleParty/gmd:individualName/gco:CharacterString" />
                </xsl:variable>
                <xsl:variable name="contact_email">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:pointOfContact/gmd:CI_ResponsibleParty/gmd:contactInfo/gmd:CI_Contact/gmd:address/gmd:CI_Address/gmd:electronicMailAddress/gco:CharacterString" />
                </xsl:variable>
                <xsl:variable name="keyWords">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:descriptiveKeywords/gmd:MD_Keywords/gmd:keyword/gco:CharacterString" />
                </xsl:variable>
                <xsl:variable name="abstract">
                    <xsl:value-of select="./gmd:MD_Metadata/gmd:identificationInfo/gmd:MD_DataIdentification/gmd:abstract/gco:CharacterString" />
                </xsl:variable>
                <!-- contents -->
                <table class="md">
                    <tr>
                        <td class="key">Catalogue:</td>
                        <td class="val"><xsl:value-of select="$CATALOG"/></td>
                    </tr>
                    <tr>
                        <td class="key">Topic Category:</td>
                        <td class="val"><xsl:value-of select="$topicCategory"/></td>
                    </tr>
                    <tr>
                        <td class="key">Identifier:</td>
                        <td class="val"><xsl:value-of select="$ident"/></td>
                    </tr>
                    <tr>
                        <td class="key">Title:</td>
                        <td class="val"><xsl:value-of select="$title"/></td>
                    </tr>
                    <tr>
                        <td class="key">Metadata Type:</td>
                        <td class="val"><xsl:value-of select="$mdType"/></td>
                    </tr>
                    <tr>
                        <td class="key">BoundingBox:</td>
                        <td class="val"><xsl:value-of select="$minx"/>, <xsl:value-of select="$miny"/>, <xsl:value-of select="$maxx"/>, <xsl:value-of select="$maxy"/></td>
                    </tr>
                    <tr>
                        <td class="key">Creation Date:</td>
                        <td class="val"><xsl:value-of select="$creation_date"/></td>
                    </tr>
                    <tr>
                        <td class="key">Contact:</td>
                        <td class="val"><xsl:value-of select="$contact"/></td>
                    </tr>
                    <tr>
                        <td class="key">Contact Email:</td>
                        <td class="val"><xsl:value-of select="$contact_email"/></td>
                    </tr>
                    <tr>
                        <td class="key">Keywords:</td>
                        <td class="val"><xsl:value-of select="$keyWords"/></td>
                    </tr>
                    <!-- 
                    <tr>
                        <td class="key">Abstract:</td>
                        <td class="val"><xsl:value-of select="$abstract"/></td>
                    </tr>
                    -->
                </table>
            </xsl:otherwise>
        </xsl:choose>
        <!-- control elements -->
        <table class="spacetop" width="500px">
            <tr>
                <td width="30%" align="middle">
                    <a class="menu" href="javascript:getMetadataOverview();"><!-- <img src="./images/csw_overview.gif" border="0" /> -->OVERVIEW</a>
                </td>
                <!-- 
                <td width="40%" align="middle">
					<xsl:choose>
					<xsl:when test="$ServLength &gt; 0">
					<a class="title" href="{$shoppingRef}"><img src="./images/csw_add_to_choice.gif" border="0"/> ADD TO SHOPPING CART</a>
					</xsl:when>
					<xsl:otherwise>
					<xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
					</xsl:otherwise>
					</xsl:choose>
				</td>
                -->
                <td width="30%" align="middle">
                    <a class="menu" href="javascript:window.close();"><!-- <img src="./images/csw_close.gif" border="0" /> -->CLOSE</a>
                </td>
                <td>
                    <xsl:choose>
                        <xsl:when test="$mdType='series'"><a href="{$childListRef}">CHILDREN</a></xsl:when>
                	</xsl:choose>
            	</td>
            </tr>
        </table>
    </xsl:template>
</xsl:stylesheet>
