<?xml version="1.0" encoding="UTF-8"?>
<!-- This file is part of deegree, for copyright/license information, please visit http://www.deegree.org/license. -->
<xsl:stylesheet version="1.0"
 xmlns="http://www.opengis.net/context"
 xmlns:csw="http://www.opengis.net/cat/csw/2.0.2"
 xmlns:fn="http://www.w3.org/2005/02/xpath-functions"
 xmlns:ogc="http://www.opengis.net/ogc"
 xmlns:xlink="http://www.w3.org/1999/xlink"
 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
 xmlns:gco="http://www.isotc211.org/2005/gco" 
 xmlns:gmd="http://www.isotc211.org/2005/gmd" 
 xmlns:srv="http://www.isotc211.org/2005/srv" >

    <xsl:output method="html" media-type="text/html" omit-xml-declaration="yes" encoding="UTF-8" />

    <xsl:param name="CATALOG"></xsl:param>
    <xsl:param name="HITS"></xsl:param> <!-- number of hits for this catalog -->
    <xsl:param name="STARTPOS"></xsl:param> <!-- startPosition for the current metaList -->

    <xsl:variable name="quote"><xsl:text disable-output-escaping="yes">'</xsl:text></xsl:variable>

    <xsl:template match="ogc:ServiceExceptionReport">
        <table class="except spacetop">
            <tr>
                <td class="key">Error</td>
                <td class="val"><xsl:value-of select="./ogc:ServiceException"/></td>
            </tr>
        </table>
    </xsl:template>

    <xsl:template match="csw:GetRecordsResponse">
        <xsl:apply-templates select="csw:SearchResults"/>
    </xsl:template>

    <xsl:template match="csw:SearchResults">
        <xsl:variable name="recReturned">
            <xsl:value-of select="@numberOfRecordsReturned"/>
        </xsl:variable>
        <xsl:variable name="argsPrev" select="concat( $quote, $CATALOG, $quote, ', ', $HITS, ', ', $recReturned, ', ', $quote, 'previous', $quote )" />
        <xsl:variable name="argsNext" select="concat( $quote, $CATALOG, $quote, ', ', $HITS, ', ', $recReturned, ', ', $quote, 'next', $quote )" />
        <xsl:variable name="previousPage" select="concat( 'javascript:turnPage( ', $argsPrev, ' );' )"/>
        <xsl:variable name="nextPage" select="concat( 'javascript:turnPage( ', $argsNext, ' );' )"/>
        <table class="md spacetop">
            <tr>
                <td class="key" colspan="2">Catalogue <xsl:value-of select="$CATALOG"/> has <xsl:value-of select="$HITS"/> results.</td>
            </tr>
            <xsl:if test="$recReturned > 0">
            <tr>
                <td class="val" colspan="2">
                    <ul><xsl:apply-templates select="gmd:MD_Metadata"/></ul>
                </td>
            </tr>
            <tr>
                <td width="50%" align="right">
                    <xsl:choose>
                        <xsl:when test="$STARTPOS &gt; 1">
                            <a class="title" href="{$previousPage}"><!-- <img src="./images/csw_previous.gif" border="0" alt="prevpage" /> -->PREVIOUS</a>
                            <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </td>
                <td width="50%" align="left">
                    <xsl:choose>
                        <xsl:when test="($STARTPOS + $recReturned) &lt;= $HITS">
    	                    <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
                            <a class="title" href="{$nextPage}"><!-- <img src="./images/csw_next.gif" border="0" alt="nextpage" /> -->NEXT</a>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </td>
            </tr>
            </xsl:if>
        </table>
    </xsl:template>

	<xsl:template match="gmd:MD_Metadata">
		<xsl:variable name="ident">
			<xsl:value-of select="./gmd:fileIdentifier/gco:CharacterString" />
		</xsl:variable>
		<xsl:variable name="mdType">
        	<xsl:value-of select="./gmd:hierarchyLevel/gmd:MD_ScopeCode/@codeListValue"/>
        </xsl:variable>
		<xsl:choose>
		    <!-- In case the metadata type is service metadata -->
			<xsl:when test="$mdType = 'service'">
				<xsl:variable name="title">
					<xsl:value-of select="./gmd:identificationInfo/srv:SV_ServiceIdentification/gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString" />
				</xsl:variable>
				<xsl:variable name="minx">
					<xsl:value-of
						select="./gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="maxx">
					<xsl:value-of
						select="./gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="miny">
					<xsl:value-of
						select="./gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="maxy">
					<xsl:value-of
						select="./gmd:identificationInfo/srv:SV_ServiceIdentification/srv:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="args" select="concat( $quote, $ident, $quote, ', ', $quote, $CATALOG, $quote, ', ', $minx, ', ', $miny, ', ', $maxx, ', ', $maxy, ', ', $quote, 'showMetadataOverview', $quote )" />
				<xsl:variable name="href" select="concat( 'javascript:getMetadata( ', $args, ' )' )" />
		        <li>
			        <a href="{$href}"> <xsl:value-of select="$title" /></a>
		        </li>
			</xsl:when>
			<xsl:otherwise>
			    <!-- In case the metadata type is dataset, series or application-->
				<xsl:variable name="title">
					<xsl:value-of select="./gmd:identificationInfo/gmd:MD_DataIdentification/gmd:citation/gmd:CI_Citation/gmd:title/gco:CharacterString" />
				</xsl:variable>
				<xsl:variable name="minx">
					<xsl:value-of select="./gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:westBoundLongitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="maxx">
					<xsl:value-of select="./gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:eastBoundLongitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="miny">
					<xsl:value-of select="./gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:southBoundLatitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="maxy">
					<xsl:value-of select="./gmd:identificationInfo/gmd:MD_DataIdentification/gmd:extent/gmd:EX_Extent/gmd:geographicElement/gmd:EX_GeographicBoundingBox/gmd:northBoundLatitude/gco:Decimal" />
				</xsl:variable>
				<xsl:variable name="args" select="concat( $quote, $ident, $quote, ', ', $quote, $CATALOG, $quote, ', ', $minx, ', ', $miny, ', ', $maxx, ', ', $maxy, ', ', $quote, 'showMetadataOverview', $quote )" />
				<xsl:variable name="href" select="concat( 'javascript:getMetadata( ', $args, ' )' )" />
				<li>
				    <a href="{$href}"><xsl:value-of select="$title" /></a>
				</li>
            </xsl:otherwise>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet>
