<!-- This file is part of deegree, for copyright/license information, please visit http://www.deegree.org/license. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:cntxt="http://www.opengis.net/context" xmlns:sld="http://www.opengis.net/sld" xmlns:deegree="http://www.deegree.org/context" xmlns:xlink="http://www.w3.org/1999/xlink">
	<xsl:template match="deegree:Module">
		<xsl:if test="./@hidden != 'true'">
       <!-- handle pushbutton/menu modules -->
			<xsl:if test="./@type = 'menu'">
				<p>
					<xsl:attribute name="id">BT<xsl:value-of select="./deegree:Name"/>
					</xsl:attribute>
				</p>
			</xsl:if>
        <!-- handle toolbar modules -->
			<xsl:if test="./@type = 'toolbar'">
				<iframe marginwidth="0" marginheight="0" frameborder="0" scrolling="no">
					<xsl:attribute name="id">
						<xsl:value-of select="./deegree:Name"/>
					</xsl:attribute>
					<xsl:attribute name="src">
						<xsl:value-of select="./deegree:Content"/>
					</xsl:attribute>
					<xsl:if test="local-name(..) = 'North'">
						<xsl:attribute name="height">
							<xsl:value-of select="$vNorthHeight"/>
						</xsl:attribute>
					</xsl:if>
					<xsl:if test="local-name(..) = 'South'">
						<xsl:attribute name="height">
							<xsl:value-of select="$vSouthHeight"/>
						</xsl:attribute>
					</xsl:if>
                <!-- use usedefined width if available -->
					<xsl:choose>
						<xsl:when test="./@width">
							<xsl:attribute name="width">
								<xsl:value-of select="./@width"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="width">
								<xsl:value-of select="(count(./deegree:ParameterList/deegree:Parameter)-1)*30 + 10"/>
							</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="./@height">
							<xsl:attribute name="height">
								<xsl:value-of select="./@height"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="height">30</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
				</iframe>
			</xsl:if>
        <!-- handle spaceing modules for adjusting the postion of other modules -->
			<xsl:if test="./@type = 'space'">
				<img src="./images/space.gif" border="0">
					<xsl:choose>
						<xsl:when test="./@width">
							<xsl:attribute name="width">
								<xsl:value-of select="./@width"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="width">1</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="./@height">
							<xsl:attribute name="height">
								<xsl:value-of select="./@height"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="height">1</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
				</img>
            <!-- force line break if space is placed in West or East area -->
				<xsl:if test="local-name(..) = 'West'">
					<br/>
				</xsl:if>
				<xsl:if test="local-name(..) = 'East'">
					<br/>
				</xsl:if>
			</xsl:if>
        <!-- handle modules offering content to the client by inserting an IFRAME -->
			<xsl:if test="./@type = 'content'">

				<iframe marginheight="0" marginwidth="0" frameborder="0" >
					<xsl:attribute name="id">
						<xsl:value-of select="./deegree:Name"/>
					</xsl:attribute>
					<xsl:attribute name="src">
						<xsl:value-of select="./deegree:Content"/>
					</xsl:attribute>
					<xsl:choose>
						<xsl:when test="./@scrolling != ''">
							<xsl:attribute name="scrolling">
								<xsl:value-of select="./@scrolling"/>
							</xsl:attribute>
						</xsl:when>
						<xsl:otherwise>
							<xsl:attribute name="scrolling">auto</xsl:attribute>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:if test="local-name(..) = 'Center'">
                        <!-- use usedefined width if available, otherwise use global defined width -->
						<xsl:choose>
							<xsl:when test="../@width">
								<xsl:attribute name="width">
									<xsl:value-of select="../@width"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="width">
									<xsl:value-of select="$vCenterWidth"/>
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
                        <!-- use usedefined height if available, otherwise use global defined height -->
						<xsl:choose>
							<xsl:when test="./@height">
								<xsl:attribute name="height">
									<xsl:value-of select="./@height"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="height">
									<xsl:value-of select="$vCenterHeight"/>
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:if>
					<xsl:if test="local-name(..) = 'North'">
						<xsl:choose>
							<xsl:when test="../@height != '0' ">
								<xsl:attribute name="height">
									<xsl:value-of select="../@height"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="height">
									<xsl:value-of select="$vNorthHeight"/>
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>
                        <!-- use usedefined width if available -->
						<xsl:if test="./@width">
							<xsl:attribute name="width">
								<xsl:value-of select="./@width"/>
							</xsl:attribute>
						</xsl:if>
					</xsl:if>
					<xsl:if test="local-name(..) = 'South'">
						<xsl:choose>
							<xsl:when test="../@height != '0' ">
								<xsl:attribute name="height">
									<xsl:value-of select="../@height"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="height">
									<xsl:value-of select="$vSouthHeight"/>
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose> 						
                        <!-- use usedefined width if available -->
						<xsl:if test="./@width">
							<xsl:attribute name="width">
								<xsl:value-of select="./@width"/>
							</xsl:attribute>
						</xsl:if>
					</xsl:if>
					<xsl:if test="local-name(..) = 'West'">
						<xsl:choose>
							<xsl:when test="../@width != '0' ">
								<xsl:attribute name="width">
									<xsl:value-of select="../@width"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="width">
									<xsl:value-of select="$vWestWidth"/>
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>    						
                        <!-- use usedefined height if available -->
						<xsl:if test="./@height">
							<xsl:attribute name="height">
								<xsl:value-of select="./@height"/>
							</xsl:attribute>
						</xsl:if>
					</xsl:if>
					<xsl:if test="local-name(..) = 'East'">
						<xsl:choose>
							<xsl:when test="../@width != '0' ">
								<xsl:attribute name="width">
									<xsl:value-of select="../@width"/>
								</xsl:attribute>
							</xsl:when>
							<xsl:otherwise>
								<xsl:attribute name="width">
									<xsl:value-of select="$vEastWidth"/>
								</xsl:attribute>
							</xsl:otherwise>
						</xsl:choose>    
                        <!-- use usedefined height if available -->
						<xsl:if test="./@height">
							<xsl:attribute name="height">
								<xsl:value-of select="./@height"/>
							</xsl:attribute>
						</xsl:if>
					</xsl:if>
				</iframe>
			</xsl:if>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
