<!-- This file is part of deegree, for copyright/license information, please visit http://www.deegree.org/license. -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:cntxt="http://www.opengis.net/context" xmlns:sld="http://www.opengis.net/sld" xmlns:deegree="http://www.deegree.org/context" xmlns:xlink="http://www.w3.org/1999/xlink">
<xsl:template match="cntxt:ViewContext">
<html>
    <head>
        <title>
            <xsl:value-of select="./cntxt:General/cntxt:Title"/>
        </title>
        <link rel="stylesheet" type="text/css">
            <xsl:attribute name="href"><xsl:value-of select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:Style"/></xsl:attribute>
        </link>
        <xsl:apply-templates select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:CommonJS/deegree:Name"/>
        <xsl:apply-templates select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module/deegree:ModuleJS"/>
        <script language="JavaScript1.2" type="text/javascript" src="history.js"></script>
        <script language="JavaScript1.2" type="text/javascript" src="screenswitcher.js"></script>
        <script language="JavaScript1.2" type="text/javascript" src="./javascript/rpc.js"></script>
        <script language="JavaScript1.2" type="text/javascript" src="./javascript/dojo-1.4.1/dojo/dojo.js"></script>
            
            <script type="text/javascript">
             dojo.require("dojox.gfx");
            </script>
        <script language="JavaScript">

        var controller = null;
        var repository = new Array();
        repository[0] = new Array();

        if ( controller == null ) {
          controller = new Controller();
          controller.init();
        }
        
        function addObjectToRepository( key, value ) {
          repository[0][key] = value;
        }
        
        function getObjectFromRepository(key) {
          return repository[0][key];
        }
        
        function closeAreaWindow(id) {        
			if ( id.indexOf( '_closed' ) &gt; 1 ) {
				var div = document.getElementById( id );
				div.style.visibility = "hidden";
				div = document.getElementById( id.substring( 0, id.length - 7 ) );
				div.style.visibility = "visible";
			} else {
				var div = document.getElementById(  id );
				div.style.visibility = "hidden";
				div = document.getElementById( id + '_closed' );
				div.style.visibility = "visible";
			}
        }

        function Controller() {
            this.mapModel = null;
            // object for transforming map coordinated to pixel coordinates
            // and vice versa
            this.gtrans = null;
            this.vMapController = null;
            this.mode = null;
            this.init = init;
            this.layerList = null;
            this.history = new History( this, 100 );
            this.isFullScreen = isFullScreen;
            this.initMapModel = initMapModel;
            this.actionPerformed = actionPerformed;
            this.performZoom = performZoom;
            this.performZoomByPoint = performZoomByPoint;
            this.performZoomByRect = performZoomByRect;
            this.performFeatureInfo = performFeatureInfo;
            this.performRecenter = performRecenter;
            this.performMove = performMove;
            this.repaint = repaint;
            this.replace = replace;
            this.addLayersToModel = addLayersToModel;
            this.setBBOXToFullMap = setBBOXToFullMap;
            this.openDownloadDialog = openDownloadDialog;
            this.removeLayerGroup = removeLayerGroup;
            this.moveLayerGroupUp = moveLayerGroupUp;
            this.moveLayerGroupDown = moveLayerGroupDown;
            this.resetContext = resetContext;

            this.setMapScreenCoords = setMapScreenCoords;
            this.getFeatureTypes = getFeatureTypes;
            function replace(page) {
                window.location.replace(page);
            }

            /*
             * adds a list of layers served by one WMS to the map model. the passed layer
             * parameter is an associative array with one field for each layer. each layer
             * contains 'name', 'title', 'queryable'
             */
            function addLayersToModel(wmsname, url, version, layers, format) {
                var wmslayer  = new Array();
                var lay = null;
                for (var i = 0; i &lt; layers.length; i++) {
                    /*
                    WMSLayer( wmsName, name, title, layerAbstract, 
                              styleName, SLDRef, visible, selected, queryable, minScale, 
                              maxScale, legendURL, metadataURL, dsResource, dsGeomType, dsFeatureType )
                    */
                    lay = new WMSLayer( wmsname, layers[i]['name'], layers[i]['title'], layers[i]['abstract'], 
                                        'default', null, false, false, layers[i]['queryable'], layers[i]['minScale'], 
                                        layers[i]['maxScale'], null, layers[i]['metadataURL'], null, null, null );
                    wmslayer.push( lay );
                }
                var lg = new LayerGroup(wmsname, 'OGC:WMS ' + version,  wmsname, url, wmslayer);
                lg.format=format;
                var ll = this.mapModel.getLayerList();
                ll.addLayerGroup( lg );
                this.mapModel.setChanged( true );
                this.repaint();
            }

            function removeLayerGroup(id) {
                var ll = this.mapModel.getLayerList();
                this.mapModel.setChanged( true );
                ll.removeLayerGroupById( id );
                this.repaint();
            }

            function moveLayerGroupUp(index) {
                var ll = this.mapModel.getLayerList();
                this.mapModel.setChanged( true );
                ll.swapLayerGroupOrder( index, index-1 );
                this.repaint();
            }

            function moveLayerGroupDown(index) {
                var ll = this.mapModel.getLayerList();
                this.mapModel.setChanged( true );
                ll.swapLayerGroupOrder( index, index+1 );
                this.repaint();
            }

            function getModeName(param) {
                var sp = param.split('|');
                return sp[0];
            }

            function getTooltip(param) {
                var sp = param.split('|');
                if ( sp.length > 1 ) {
                    return sp[1];
                } else {
                    return sp[0];
                }
            }

            function setBBOXToFullMap() {
                this.mapModel.setBoundingBox( this.mapModel.getInitialBoundingBox() );
            }

            /* download of WFS features in current bounding box for registered and unregistered users */
            function openDownloadDialog() {
            	var featureTypes = this.getFeatureTypes();
                if ( featureTypes == null || featureTypes == "" ) {
                    alert( "The selected layer does not contain available data to download" );
                    return;
                }
                var sessionID = null;
                if ( this.vSessionKeeper != null &amp;&amp; this.vSessionKeeper.id != null ) {
                    sessionID = this.vSessionKeeper.id;
                }
                var win = window.open( 'control?action=mapView:initDownload&amp;sessionID=' + sessionID, 'download',
                                       'width=550,height=400,left=100,top=0,scrollbars=yes,resizable=yes' );
                win.focus();
            }

			function getFeatureTypes() {
                var lList = this.mapModel.getLayerList();
                var lGroups = lList.getLayerGroups();
                var featureTypes = "";
                
                for( var i = 0; i &lt; lGroups.length; i++ ){
                    var layers = lGroups[i].getLayers();
                    for( var j = 0; j &lt; layers.length; j++ ){
                        //both FeatureType and GeometryType must be known for the layer to be downloadable
                        if ( layers[j].getDSFeatureType() != null &amp;&amp; layers[j].getDSFeatureType() != "" &amp;&amp;
                             layers[j].getDSGeomType() != null &amp;&amp; layers[j].getDSGeomType() != "" ) {
                                 
                            var v = layers[j].getDSFeatureType();
                            if ( j &lt; layers.length-1 ) {
                                v += ",";
                            }
                            featureTypes += v;
                        }
                    }
                }
                return featureTypes;
            }
            
            function resetContext() {
                var req = "control?rpc=&lt;?xml version='1.0' encoding='UTF-8'?&gt;&lt;methodCall&gt;" +
                         "&lt;methodName&gt;mapClient:resetContext&lt;/methodName&gt;&lt;/methodCall&gt;";
                this.replace(req);
            }

            function actionPerformed(event) {
				this.mapModel.setWidth( <xsl:value-of select="$vMapWidth"/> );
				this.mapModel.setHeight( <xsl:value-of select="$vMapHeight"/> );       
                this.initMapHandling( <xsl:value-of select="$vBorder"/>,<xsl:value-of select="$vBorder"/>, '#FF0000' );
                                  
                <xsl:for-each select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module">                 
                
                    <xsl:if test="./@type = 'toolbar'">
                        if ( this.vMapEditor ) {
                            this.vMapEditor.tb.reset();
                        }
                        <xsl:for-each select="./deegree:ParameterList/deegree:Parameter">
                            <xsl:if test="deegree:Name != 'selected'">
                                <xsl:if test="deegree:Name != 'bgcolor'">
                                    if ( event.name == ( getModeName('<xsl:value-of select="deegree:Name"/>') + 'Pressed') ) {
                                    <xsl:if test="deegree:Action">
                                        <!-- if a specific action is defined to be performed when pressing a toggle button print it here -->
                                        <xsl:value-of select="deegree:Action"/>
                                    </xsl:if>
                                    <!-- if a toggle button has been pressed, store the assigned mode -->
                                    <xsl:if test="./deegree:Value = 'ToggleButton'">this.mode = event.value;</xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'fullextent' )">this.setBBOXToFullMap(); this.history.addEnvelope( this.mapModel.getBoundingBox() ); this.repaint();</xsl:if>

                                    <xsl:if test="starts-with( ./deegree:Name, 'movetoprevious' )">this.history.moveBack();</xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'movetonext' )">this.history.moveForward();</xsl:if>

                                    <xsl:if test="starts-with( ./deegree:Name, 'home' )">this.resetContext();</xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'refresh')">this.repaint();</xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'addwms')">
                                        var win = window.open( './addwms.jsp' ,'add', 'width=650,height=400,left=100,top=0,scrollbars=yes,resizable=yes');
                                        //var win = window.open( './addwms.html' ,'add', 'width=650,height=400,left=100,top=0,scrollbars=yes,resizable=yes');
                                        win.focus();
                                    </xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'print')">
                                        var win = window.open( './printviewopener.html' ,'print', 'width=650,height=500,left=100,top=0,scrollbars=yes,resizable=yes');
                                        win.focus();
                                    </xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'pdfprint')">
                                        var win = window.open( './modules/pdfprint/printdialog.jsp', 'pdfprint', 'width=850,height=600,left=100,top=0,scrollbars=yes,resizable=yes');
                                        win.focus();
                                    </xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'download')">
                                        this.openDownloadDialog();
                                    </xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'zoom2layer')">
                                        doRecenterToLayerRequest( document, controller );
                                    </xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'fullScreen' )">
                                    	switchScreen("mapClient:fullScreen");
                                    </xsl:if>
                                    <xsl:if test="starts-with( ./deegree:Name, 'normalScreen' )">
                                    	switchScreen("mapClient:normalScreen");
                                    </xsl:if>
                                } else
                                </xsl:if>
                            </xsl:if>
                        </xsl:for-each>
                        if ( event.name == 'PAN' ) {
                            this.vMapController.pan( event.value, 25);
                            this.history.addEnvelope( this.mapModel.getBoundingBox() );
                            this.repaint();
                        } else if ( event.name == 'BOX' &amp;&amp;  (this.mode == 'zoomin' || this.mode == 'zoomout') ) {
                            this.performZoom(event);
                            this.history.addEnvelope( this.mapModel.getBoundingBox() );
                        }  else if ( event.name == 'BOX' &amp;&amp;  (this.mode == 'featureinfo') ) {
                            this.performFeatureInfo(event);
                        } else if ( event.name == 'BOX' &amp;&amp;  (this.mode == 'recenter') ) {
                            this.performRecenter(event);
                            this.history.addEnvelope( this.mapModel.getBoundingBox() );
                        }  else if ( event.name == 'BBOX' ) {
                            var env1 = new Envelope( event.value.minx, event.value.miny,
                                                     event.value.maxx, event.value.maxy );
                            this.mapModel.setBoundingBox( env1 );
                            this.mapModel = this.vMapController.ensureAspectRatio();
                            this.history.addEnvelope( this.mapModel.getBoundingBox() );
                            this.repaint();
                        }  else if ( event.name == 'MOVE' ) {
                            this.performMove( event );
                            this.history.addEnvelope( this.mapModel.getBoundingBox() );
                        } else {
                            alert('unknown event: ' + event.name + ' - ' + this.mode);
                        }
                    </xsl:if>
                </xsl:for-each>
            }

			function isFullScreen() {
                <xsl:for-each select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module">
                    <xsl:if test="./@type = 'toolbar'">
                        <xsl:for-each select="./deegree:ParameterList/deegree:Parameter">
                            <xsl:if test="deegree:Name != 'selected'">
                                <xsl:if test="starts-with( ./deegree:Name, 'fullScreen' )">
                                	return false;
                                </xsl:if>
                                <xsl:if test="starts-with( ./deegree:Name, 'normalScreen' )">
                                	return true;
                                </xsl:if>
                            </xsl:if>
                        </xsl:for-each>
                     </xsl:if>
                </xsl:for-each>     
			}
			
            function performZoom(event) {
                var env = event.value;
                if ( (env.maxx - env.minx &lt; 5 &amp;&amp; env.maxy - env.miny &lt; 5) ||
                    this.mode == 'zoomout' ) {
                    this.performZoomByPoint( env );
                } else {
                    this.performZoomByRect( env );
                }
                this.repaint();
            }

            function performZoomByPoint(envelope) {
                var level = 25;
                if ( this.mode == 'zoomin' ) {
                    level = level * -1;
                }
                //level = level &lt; 0 ? level/100.0 + 1.0 : 1.0 / (1.0 - (level/100.0));
                this.vMapController.zoomByPoint( level, this.gtrans.getSourceX(envelope.minx),
                                                 this.gtrans.getSourceY(envelope.miny) );
            }

            function performZoomByRect(envelope) {
                var mnx = this.gtrans.getSourceX(envelope.minx);
                var mxy = this.gtrans.getSourceY(envelope.miny);
                var mxx = this.gtrans.getSourceX(envelope.maxx);
                var mny = this.gtrans.getSourceY(envelope.maxy);
                env = new Envelope( mnx, mny, mxx, mxy );
                this.mapModel.setBoundingBox( env );
                this.mapModel = this.vMapController.ensureAspectRatio();
            }

            function performFeatureInfo(event) {
				var fact = new WMSRequestFactory();
				var lgs = this.mapModel.getLayerList().getLayerGroups();
				var fi = false;
				tmp = '';
				for ( var i = 0; i &lt; lgs.length; i++) {
					var s = null;
					if ( controller.vSessionKeeper != null &amp;&amp; controller.vSessionKeeper.id != null ) {
						s = fact.createGetFeatureInfoRequest( lgs[i], this.mapModel, "text/html",
															  event.value.minx, event.value.miny,
															  controller.vSessionKeeper.id);
					} else {
						s = fact.createGetFeatureInfoRequest( lgs[i], this.mapModel, "text/html",
															  event.value.minx, event.value.miny, null);
					}
					if ( s != null &amp;&amp; s.indexOf( "ERROR: " ) &lt; 0 ) {
						fi = true;
						var fiw = window.open( s ,"FeatureInfo","width=550,height=400,left=0,top=0,scrollbars=yes");
						fiw .focus();
					} else {
					  tmp += s;
					}
				}
				if ( !fi ) {
					alert( tmp );
				}
			}

            function performRecenter(event) {
                var env = event.value;
                var mEnv = this.mapModel.getBoundingBox();
                var level = 1;
                this.vMapController.zoomByPoint( level, this.gtrans.getSourceX(env.minx),
                                                 this.gtrans.getSourceY(env.miny) );
                this.repaint();
            }

            function performMove(event) {
                var env = event.value;
                var mEnv = this.mapModel.getBoundingBox();
                var envelope = this.mapModel.getBoundingBox();
                this.gtrans = new GeoTransform( mEnv.minx, mEnv.miny, mEnv.maxx, mEnv.maxy, 0, 0,
                                                this.mapModel.getWidth()-1, this.mapModel.getHeight()-1 );
                var xmin = this.gtrans.getSourceX( -1*env[0] ) ;
                var ymax = this.gtrans.getSourceY( -1*env[1] ) ;
                var bb = new Envelope( xmin, ymax-(mEnv.maxy-mEnv.miny), xmin+ (mEnv.maxx-mEnv.minx), ymax );
                this.mapModel.setBoundingBox( bb );
                this.repaint();
            }

            function setMapScreenCoords( x, y ) {
                if ( this.vCoordinateDisplay ) {
                    x = this.gtrans.getSourceX( x );
                    y = this.gtrans.getSourceY( y );
                    this.vCoordinateDisplay.setCoordinatesAtCursor( x, y );
                }
            }

			function switchScreen( methodName ) {
				var req = new ScreenSwitcher().switchToFullScreen( methodName );
               	document.getElementById( 'hiddenScaleScreen' ).value = req;
			    document.getElementById( 'scaleScreen' ).submit();
			}

            function repaint() {
                // update with current map size
                var envelope = this.mapModel.getBoundingBox();
                this.gtrans = new GeoTransform( envelope.minx, envelope.miny, envelope.maxx,
                                                envelope.maxy, 0, 0, this.mapModel.getWidth()-1,
                                                this.mapModel.getHeight()-1 );

                <xsl:for-each select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module">
                    <xsl:if test="./@type != 'menu'">
                        <xsl:if test="./@type != 'space'">
                            if ( this.v<xsl:value-of select="./deegree:Name"/> != null ) {
                                this.v<xsl:value-of select="./deegree:Name"/>.repaint();
                            }
                        </xsl:if>
                    </xsl:if>
                </xsl:for-each>
            }

            <!-- declare a variable and an initMethod for each module that is not of type 'menu' and not of type 'space'-->
            <xsl:for-each select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module">
                <xsl:if test="./@type != 'menu'">
                    <xsl:if test="./@type != 'space'">
                        this.v<xsl:value-of select="./deegree:Name"/> = null;
                        this.init<xsl:value-of select="./deegree:Name"/> = init<xsl:value-of select="./deegree:Name"/>;
                    </xsl:if>
                </xsl:if>
            </xsl:for-each>
            function init() {
                this.initMapModel();
                this.history.init( this.mapModel.getBoundingBox() );
                this.vMapController = new MapController(parent.controller.mapModel);
                <xsl:for-each select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module">
                    addObjectToRepository( 'SRC:' + '<xsl:value-of select="./deegree:Name"/>', '<xsl:value-of select="./deegree:Content"/>' );
                </xsl:for-each>
            }

            <!-- implement initMethod for each module that is of type 'content' or 'toolbar'-->
            <xsl:for-each select="./cntxt:General/cntxt:Extension/deegree:Frontend/deegree:*/deegree:Module">
                <xsl:if test="./@type = 'content' or ./@type = 'overlay' ">
                    function init<xsl:value-of select="./deegree:Name"/>(doc) {
                        <xsl:variable name="vParam">
                            <xsl:for-each select="./deegree:ParameterList/deegree:Parameter/deegree:Value">
                                <xsl:value-of select="."/>,</xsl:for-each>
                        </xsl:variable>
                        if ( this.v<xsl:value-of select="./deegree:Name"/>  == null ) {
                        this.v<xsl:value-of select="./deegree:Name"/> =
                            new <xsl:value-of select="./deegree:Name"/>( <xsl:value-of select="substring( $vParam, 0, string-length($vParam) )"/> );
                        }
                        this.v<xsl:value-of select="./deegree:Name"/>.paint( doc, doc.getElementsByTagName('body')[0] );
                    }
                </xsl:if>
                <xsl:if test="./@type = 'toolbar'">
                    function init<xsl:value-of select="./deegree:Name"/>(doc) {
                        var bt = null;
                        var sn = null;
                        var sp = null;
                        var buttonGroups = new Array( new ButtonGroup('<xsl:value-of select="./deegree:Name"/>') );
                        <xsl:for-each select="./deegree:ParameterList/deegree:Parameter">
                            <xsl:if test="deegree:Name != 'selected'">
                                <xsl:if test="deegree:Name != 'bgcolor'">
                                    sn = getModeName('<xsl:value-of select="deegree:Name"/>');
                                    sp = getTooltip('<xsl:value-of select="deegree:Name"/>');
                                    bt = new <xsl:value-of select="deegree:Value"/>(sn, sp, './images/' + sn+ '.gif', './images/' + sn + '_a.gif' );
                                    bt.setListener( controller );
                                    buttonGroups[0].addButton( bt );
                                </xsl:if>
                            </xsl:if>
                        </xsl:for-each>
                        <!-- JM: changes to enable ommitting param bgcolor in toolbar module configuration -->
                        <xsl:variable name="vTBCol">
                            <xsl:choose>
                                <xsl:when test="boolean(deegree:ParameterList/deegree:Parameter[deegree:Name='bgcolor'])">
                                    <xsl:value-of select="deegree:ParameterList/deegree:Parameter[deegree:Name='bgcolor']/deegree:Value" />
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="'null'" />
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        
                        this.v<xsl:value-of select="./deegree:Name"/> =
                            new Toolbar('<xsl:value-of select="./deegree:Name"/>', buttonGroups,
                                         <xsl:value-of select="$vMapWidth"/>, <xsl:value-of select="$vNorthHeight"/>,
                                         <xsl:value-of select="$vTBCol"/>);
                        this.v<xsl:value-of select="./deegree:Name"/>.paint( doc, doc.getElementsByTagName('body')[0] );
                    }
                </xsl:if>
            </xsl:for-each>

                    <!-- imlement initMapModel Method -->
            function initMapModel() {
                var layerGroups = new Array();
                var wmslayer  = new Array();
                var lay = null;
                var lg = null;
                <xsl:for-each select="./cntxt:LayerList/cntxt:Layer">
                    <xsl:variable name="vHidden">
                        <xsl:if test="./@hidden ='1'">false</xsl:if>
                        <xsl:if test="./@hidden ='true'">false</xsl:if>
                        <xsl:if test="./@hidden ='0'">true</xsl:if>
                        <xsl:if test="./@hidden ='false'">true</xsl:if>
                    </xsl:variable>
                    <xsl:variable name="vQueryable">
                        <xsl:if test="./@queryable ='1'">true</xsl:if>
                        <xsl:if test="./@queryable ='true'">true</xsl:if>
                        <xsl:if test="./@queryable ='0'">false</xsl:if>
                        <xsl:if test="./@queryable ='false'">false</xsl:if>
                    </xsl:variable>
                    <xsl:variable name="vMetadataURL">
                        <xsl:value-of select="./cntxt:MetadataURL/cntxt:OnlineResource/@xlink:href" />
                    </xsl:variable>
                    <xsl:variable name="vLegendURL">
                        <xsl:value-of select="./cntxt:StyleList/cntxt:Style[@current = 1]/cntxt:LegendURL/cntxt:OnlineResource/@xlink:href"/>
                    </xsl:variable>
                    <xsl:variable name="vMinScale">
                        <xsl:choose>
                            <xsl:when test="./cntxt:Extension/deegree:ScaleHint/@min != ''">
                                <xsl:value-of select="./cntxt:Extension/deegree:ScaleHint/@min"/>
                            </xsl:when>
                            <xsl:otherwise>0</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="vMaxScale">
                        <xsl:choose>
                            <xsl:when test="./cntxt:Extension/deegree:ScaleHint/@max != ''">
                                <xsl:value-of select="./cntxt:Extension/deegree:ScaleHint/@max"/>
                            </xsl:when>
                            <xsl:otherwise>999999999</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="vCurrentStyle">
                        <xsl:choose>
                            <xsl:when test="./cntxt:StyleList/cntxt:Style">
                                <xsl:value-of select="./cntxt:StyleList/cntxt:Style/cntxt:Name[../@current = 1]"/>
                            </xsl:when>
                            <xsl:otherwise>default</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="vSelectedForQuery">
                        <xsl:choose>
                            <xsl:when test="./cntxt:Extension/deegree:SelectedForQuery = 'true'">true</xsl:when>
                            <xsl:otherwise>false</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="vDSResource">
                        <xsl:value-of select="./cntxt:Extension/deegree:DataService/cntxt:Server/cntxt:OnlineResource/@xlink:href" />
                    </xsl:variable>
                    <xsl:variable name="vDSGeomType">
                        <xsl:value-of select="./cntxt:Extension/deegree:DataService/deegree:GeometryType" />
                    </xsl:variable>
                    <xsl:variable name="vDSFeatureType">
                        <xsl:value-of select="./cntxt:Extension/deegree:DataService/deegree:FeatureType" />
                    </xsl:variable>
                    
                    lay = new WMSLayer( '<xsl:value-of select="./cntxt:Server/@title"/>',
                                        '<xsl:value-of select="./cntxt:Name"/>',
                                        '<xsl:value-of select="./cntxt:Title"/>',
                                        '<xsl:value-of select="./cntxt:Abstract"/>',
                                        '<xsl:value-of select="$vCurrentStyle"/>', null,
                                        <xsl:value-of select="$vHidden"/>, 
                                        <xsl:value-of select="$vSelectedForQuery"/>,
                                        <xsl:value-of select="$vQueryable"/>,
                                        <xsl:value-of select="$vMinScale"/>,
                                        <xsl:value-of select="$vMaxScale"/>,
                                        '<xsl:value-of select="$vLegendURL"/>',
                                        '<xsl:value-of select="$vMetadataURL"/>',
                                        '<xsl:value-of select="$vDSResource"/>',
                                        '<xsl:value-of select="$vDSGeomType"/>',
                                        '<xsl:value-of select="$vDSFeatureType"/>' );
                    wmslayer.push( lay );
                    <xsl:variable name="vSer">
                        <xsl:value-of select="./following::*/cntxt:Server/cntxt:OnlineResource/@xlink:href"/>
                    </xsl:variable>

                    <xsl:copy-of select="./cntxt:Server/cntxt:OnlineResource/attribute"></xsl:copy-of>

                    <xsl:if test="./cntxt:Server/cntxt:OnlineResource/@xlink:href != string($vSer) or count(../cntxt:Layer) = position()">
                        <xsl:choose>
                            <xsl:when test="./cntxt:Server/cntxt:OnlineResource/@xlink:href">
                                lg = new LayerGroup( <xsl:value-of select="position()"/>,
                                                     '<xsl:value-of select="./cntxt:Server/@service"/><xsl:text> </xsl:text><xsl:value-of select="./cntxt:Server/@version"/>',
                                                     '<xsl:value-of select="./cntxt:Server/@title"/>',
                                                     '<xsl:value-of select="./cntxt:Server/cntxt:OnlineResource/@xlink:href"/>',
                                                     wmslayer);
                            </xsl:when>
                            <xsl:otherwise>
                                lg = new LayerGroup( <xsl:value-of select="position()"/>,
                                                     '<xsl:value-of select="./cntxt:Server/@service"/><xsl:text> </xsl:text><xsl:value-of select="./cntxt:Server/@version"/>',
                                                     '<xsl:value-of select="./cntxt:Server/@title"/>',
                                                     '<xsl:value-of select="./cntxt:Server/cntxt:OnlineResource/@href"/>',
                                                     wmslayer);
                            </xsl:otherwise>
                        </xsl:choose>

                        lg.setFormat('<xsl:value-of select="./cntxt:FormatList/cntxt:Format[@current='1' or @current='true']"/>');
                        layerGroups.push( lg );
                        wmslayer  = new Array();
                    </xsl:if>
                </xsl:for-each>
                this.layerList = new LayerList( 'll1' );
                for (var k = 0; k &lt; layerGroups.length; k++){
                    this.layerList.addLayerGroup(layerGroups[k]);
                }
                var envelope = new Envelope( <xsl:value-of select="$vMinx"/>,<xsl:value-of select="$vMiny"/>,
                                             <xsl:value-of select="$vMaxx"/>,<xsl:value-of select="$vMaxy"/> );
                this.mapModel = new MapModel( this.layerList, "<xsl:value-of select="$vCRS"/>", envelope,
                                              <xsl:value-of select="$vMapWidth"/>, <xsl:value-of select="$vMapHeight"/>);

                this.gtrans = new GeoTransform( envelope.minx, envelope.miny, envelope.maxx, envelope.maxy,
                                                0, 0, this.mapModel.getWidth()-1, this.mapModel.getHeight()-1 );
            }
        }
        
        function onLoadingPage() {

        }
        </script>
    </head>
    <xsl:apply-templates select="./cntxt:General/cntxt:Extension/deegree:Frontend"/>
</html>
</xsl:template>
</xsl:stylesheet>
